import Transport as tr


def con_athena_highbandwidth(database_name,
                             qry_tableName=None,
                             qry_whereClause=None,
                             qry_limitClause: str = None,
                             account_id=None,
                             ds_name=None,
                             ds_description=None,
                             query=None,
                             schedule: tr.Schedule = None
                             ):
    cnfg = tr.Config.ATHENA_HIGHBANDWIDTH.value

    if not query:
        query = f"SELECT * \nFROM {database_name}.{qry_tableName} \n{qry_whereClause} "
        if qry_limitClause:
            query += f"\n{qry_limitClause}"

    configuration = [
        tr.ConfigurationParam(
            name='queryType',
            value='customQuery'),
        tr.ConfigurationParam(
            name="dataCatalogName",
            value="AwsDataCatalog"
        ),
        tr.ConfigurationParam(
            name='databaseName',
            value=database_name),
        tr.ConfigurationParam(
            name='enteredCustomQuery',
            value=query),
        tr.ConfigurationParam(
            name='bypassDataUpload',
            value=True

        ),
        tr.ConfigurationParam(
            name="keepUnloadExecutedFiles",
            value=True)
    ]

    config = tr.Configuration(transport=tr.Transport(type=cnfg.get('type'),
                                                     description=cnfg.get(
                                                         'description'),
                                                     version=cnfg.get('version')),
                              account=tr.Account(id=account_id),
                              configuration=configuration,
                              updateMethod=tr.UpdateMethod.REPLACE.value,
                              dataProvider=tr.DataProvider(
                                  key=cnfg.get('dataProvider_key')),
                              dataSource=tr.Datasource(
                                  name=ds_name, description=ds_description),
                              advancedScheduleJson=schedule)

    return config



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/DictDot.ipynb.

# %% auto 0
__all__ = ['DictDot', 'split_str_to_obj']

# %% ../../nbs/utils/DictDot.ipynb 3
from types import SimpleNamespace

# %% ../../nbs/utils/DictDot.ipynb 4
class DictDot(SimpleNamespace):
    """a utility class that converts json into an object that supports data retrieval with dot notation"""

    def __init__(self, dictionary: dict,  # dictionary to convert
                 **kwargs):
        super().__init__(**kwargs)

        for key, value in dictionary.items():
            if isinstance(value, dict):
                self.__setattr__(key, DictDot(value))
            elif isinstance(value, list):
                new_list = []
                for item in value:
                    if isinstance(item, dict):
                        new_list.append(DictDot(item))
                    else:
                        new_list.append(item)
                self.__setattr__(key, new_list)
            else:
                self.__setattr__(key, value)

    def __getattr__(self, item):
        return None


# %% ../../nbs/utils/DictDot.ipynb 7
def split_str_to_obj(piped_str: str, key_ls: list[str]):
    """
    split a pipe separated list into an object with keys defined by the list of keys
    ex. "test_instance|myemail|sample_password", ["domo_instance", "domo_username", "domo_password"] = {"domo_instance" : "test_intance" , "domo_username" : "myemail", "domo_password":"sample_password"}
    """
    str_ls = piped_str.split('|')
    obj = dict([new_obj_key, str_ls[index]]
               for index, new_obj_key in enumerate(key_ls))

    return DictDot(obj)




import aiohttp

from ..utils.DictDot import DictDot
from dataclasses import dataclass, field

from Library.DomoClasses.DomoAuth import DomoFullAuth
from .routes import role_routes


@dataclass
class DomoRole:
    full_auth: DomoFullAuth

    id: str
    name: str = None
    description: str = None
    is_system_role: int = None

    grant_list: list[str] = field(default_factory=list)
    membership_list: list = field(default_factory=list)

    @classmethod
    def _from_str(cls, id, name, description=None, full_auth: DomoFullAuth = None):

        return cls(id=id,
                   name=name,
                   description=description,
                   full_auth=full_auth,
                   is_system_role=1 if id <= 5 else 0
                   )

    async def get_membership(self, full_auth: DomoFullAuth = None,
                             role_id=None, debug: bool = False, session: aiohttp.ClientSession = None):

        res = await role_routes.get_role_membership(full_auth=full_auth or self.full_auth,
                                                    role_id=role_id or self.id,
                                                    debug=debug, session=session)
        if res.status == 200:
            self.membership_list = res.response.get('users')

            return {'status': res.status,
                    'users': res.response.get('users')}

    @classmethod
    async def create_role(cls,
                          full_auth: DomoFullAuth,
                          name: str,
                          description: str = None,
                          debug: bool = False):

        res = await role_routes.create_role(full_auth=full_auth,
                                            name=name,
                                            description=description,
                                            debug=debug)
        if debug:
            print(res)

        if res.status == 200:
            return cls._from_str(id=res.response.get('id'),
                                 name=res.response.get('name'),
                                 description=res.response.get('description'),
                                 full_auth=full_auth)
        return None

    async def update_role_grants(self,
                                 grant_list: list[str],
                                 role_id: str = None,
                                 full_auth: DomoFullAuth = None,
                                 debug: bool = False):

        res = await role_routes.update_role_grants(full_auth=full_auth or self.full_auth,
                                                   role_id=role_id or self.id,
                                                   role_grant_list=grant_list,
                                                   debug=debug)

        if res.status == 200 or res.status == 204:
            res_roles = await role_routes.get_role_grants(full_auth or self.full_auth,
                                                          role_id=role_id or self.id,
                                                          debug=debug)
            return {'status': res.status,
                    'role_id': role_id or self.id,
                    'grants': res_roles.response
                    }

    async def add_user(self, user_list: list[str],
                       role_id: str = None,
                       full_auth: DomoFullAuth = None, debug: bool = False):

        res = await role_routes.role_membership_add_user(full_auth=full_auth or self.full_auth,
                                                         role_id=role_id or self.id,
                                                         user_list=user_list,
                                                         debug=debug)
        if res.status == 204:
            res_membership = await role_routes.get_role_membership(full_auth=full_auth or self.full_auth,
                                                                   role_id=role_id or self.id,
                                                                   debug=debug)

            return {'status': res.status,
                    'users': res_membership.response.get('users')}

        return res

    async def set_as_default_role(self, debug: bool = False):
        return await role_routes.set_default_role(full_auth=self.full_auth,
                                                  role_id=self.id,
                                                  debug=debug)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/page.ipynb.

# %% auto 0
__all__ = ['get_page_by_id', 'get_page_definition', 'get_page_access_list', 'get_pages_adminsummary', 'update_page_layout',
           'put_writelock', 'delete_writelock']

# %% ../../nbs/routes/page.ipynb 2
import httpx
import pandas as pd

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda

# %% ../../nbs/routes/page.ipynb 3
async def get_page_by_id(
    auth: dmda.DomoAuth,
    page_id: str,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
    include_layout: bool = False,
) -> rgd.ResponseGetData:
    url = f"https://{auth.domo_instance}.domo.com/api/content/v3/stacks/{page_id}/cards"

    if include_layout:
        url += "?includeV4PageLayouts=true"

    res = await gd.get_data(
        auth=auth,
        url=url,
        method="GET",
        debug_api=debug_api,
        session=session,
    )

    res.response

    return res

# %% ../../nbs/routes/page.ipynb 6
async def get_page_definition(
    auth, page_id, debug_api: bool = False, session: httpx.AsyncClient = None
):
    url = f"https://{auth.domo_instance}.domo.com/api/content/v3/stacks/{page_id}/cards"

    params = {
        "includeV4PageLayouts": "true",
        "parts": "metadata,datasources,library,drillPathURNs,certification,owners,dateInfo,subscriptions,slicers",
    }

    res = await gd.get_data(
        url,
        method="GET",
        auth=auth,
        session=session,
        params=params,
        debug_api=debug_api,
    )

    return res

# %% ../../nbs/routes/page.ipynb 9
async def get_page_access_list(
    auth,
    page_id,
    is_expand_users: bool = True,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):
    """retrieves accesslist, which users and groups a page is shared with"""

    url = f"https://{auth.domo_instance}.domo.com/api/content/v1/share/accesslist/page/{page_id}?expandUsers={is_expand_users}"

    res = await gd.get_data(
        url, method="GET", auth=auth, session=session, debug_api=debug_api
    )

    # add group members to users response
    if is_expand_users:
        group_users = [
            user for group in res.response.get("groups") for user in group.get("users")
        ]
        users = res.response.get("users") + group_users
        res.response.update({"users": users})

    return res

# %% ../../nbs/routes/page.ipynb 12
async def get_pages_adminsummary(
    auth: dmda.DomoAuth,
    debug_loop: bool = False,
    debug_api: bool = False,
    limit=35,
    session: httpx.AsyncClient = None,
):
    """retrieves all pages in instance user is able to see (but may not have been explicitly shared)"""

    url = f"https://{auth.domo_instance}.domo.com/api/content/v1/pages/adminsummary"

    offset_params = {
        "offset": "skip",
        "limit": "limit",
    }

    body = {"orderBy": "pageTitle", "ascending": True}

    def arr_fn(res) -> list[dict]:
        return res.response.get("pageAdminSummaries")

    res = await gd.looper(
        auth=auth,
        method="POST",
        url=url,
        arr_fn=arr_fn,
        offset_params=offset_params,
        session=session,
        loop_until_end=True,
        body=body,
        limit=limit,
        debug_loop=debug_loop,
        debug_api=debug_api,
    )
    return res

# %% ../../nbs/routes/page.ipynb 14
async def update_page_layout(
    auth: dmda.DomoAuth, layout_id: str, body: dict, debug_api: bool = False
):
    url = f"https://{auth.domo_instance}.domo.com/api/content/v4/pages/layouts/{layout_id}"

    res = await gd.get_data(
        auth=auth, url=url, body=body, method="PUT", debug_api=debug_api
    )

    if debug_api:
        print(res)

    return res


async def put_writelock(
    auth: dmda.DomoAuth,
    layout_id: str,
    user_id: str,
    epoch_time: int,
    debug_api: bool = False,
):
    url = f"https://{auth.domo_instance}.domo.com/api/content/v4/pages/layouts/{layout_id}/writelock"
    body = {
        "layoutId": layout_id,
        "lockHeartbeat": epoch_time,
        "lockTimestamp": epoch_time,
        "userId": user_id,
    }

    res = await gd.get_data(
        auth=auth, url=url, body=body, method="PUT", debug_api=debug_api
    )

    if debug_api:
        print(res)

    return res


async def delete_writelock(
    auth: dmda.DomoAuth, layout_id: str, debug_api: bool = False
):
    url = f"https://{auth.domo_instance}.domo.com/api/content/v4/pages/layouts/{layout_id}/writelock"
    res = await gd.get_data(auth=auth, url=url, method="DELETE", debug_api=debug_api)

    if debug_api:
        print(res)

    return res



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/stream.ipynb.

# %% auto 0
__all__ = ['get_stream_by_id', 'update_stream', 'create_stream', 'execute_stream']

# %% ../../nbs/routes/stream.ipynb 2
import httpx
import pandas as pd

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda

# %% ../../nbs/routes/stream.ipynb 3
async def get_stream_by_id(auth: dmda.DomoAuth, stream_id: str,
                                session: httpx.AsyncClient = None,
                                debug_api: bool = False) -> rgd.ResponseGetData:
    url = f'https://{auth.domo_instance}.domo.com/api/data/v1/streams/{stream_id}'

    if debug_api:
        print(url)

    res = await gd.get_data(
        auth=auth,
        url=url,
        method='GET',
        session=session,
        debug_api=debug_api,
    )
    return res


async def update_stream(auth: dmda.DomoAuth, stream_id: str,
                        body: dict,
                        session: httpx.AsyncClient = None,
                        debug_api: bool = False) -> rgd.ResponseGetData:
    url = f'https://{auth.domo_instance}.domo.com/api/data/v1/streams/{stream_id}'

    if debug_api:
        print(url)

    res = await gd.get_data(
        auth=auth,
        url=url,
        body=body,
        method='PUT',
        session=session,
        debug_api=debug_api,
    )
    return res


async def create_stream(auth: dmda.DomoAuth,
                        body: dict,
                        session: httpx.AsyncClient = None,
                        debug_api: bool = False) -> rgd.ResponseGetData:
    url = f'https://{auth.domo_instance}.domo.com/api/data/v1/streams'

    if debug_api:
        print(url)

    res = await gd.get_data(
        auth=auth,
        url=url,
        body=body,
        method='POST',
        session=session,
        debug_api=debug_api,
    )
    return res

async def execute_stream(auth: dmda.DomoAuth,
                        stream_id: str,
                        session: httpx.AsyncClient = None,
                        debug_api: bool = False) -> rgd.ResponseGetData:
    url = f'https://{auth.domo_instance}.domo.com/api/data/v1/streams/{stream_id}/executions'

    if debug_api:
        print(url)

    res = await gd.get_data(
        auth=auth,
        url=url,
        method='POST',
        session=session,
        debug_api=debug_api,
    )
    return res



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/application.ipynb.

# %% auto 0
__all__ = ['get_applications', 'get_application_by_id']

# %% ../../nbs/routes/application.ipynb 2
from typing import Union
from enum import Enum
import httpx

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de

# %% ../../nbs/routes/application.ipynb 3
async def get_applications(auth: dmda.DomoFullAuth,
                           session: Union[httpx.AsyncClient, httpx.AsyncClient, None] = None,
                           debug_api: bool = False
                           )-> rgd.ResponseGetData:

    url = f'https://{auth.domo_instance}.domo.com/api/executor/v1/applications/'

    if debug_api:
        print(url)

    return await gd.get_data(
        auth=auth,
        url=url,
        method='GET',
        debug_api=debug_api,
        session=session
    )


async def get_application_by_id(auth: dmda.DomoFullAuth,
                                application_id: str,
                                session: Union[httpx.AsyncClient, httpx.AsyncClient, None] = None,
                                debug_api: bool = False)-> rgd.ResponseGetData:

    url = f'https://{auth.domo_instance}.domo.com/api/executor/v1/applications/{application_id}'

    if debug_api:
        print(url)

    return await gd.get_data(
        auth=auth,
        url=url,
        method='GET',
        debug_api=debug_api,
        session=session
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/upload_data.ipynb.

# %% auto 0
__all__ = ['upload_data']

# %% ../../nbs/utils/upload_data.ipynb 2
import httpx
import pandas as pd

# import asyncio

import domolibrary.client.Logger as lc
import domolibrary.client.DomoAuth as dmda
import domolibrary.classes.DomoDataset as dmds


# %% ../../nbs/utils/upload_data.ipynb 3
async def loop_upload(
    upload_df: pd.DataFrame,
    consol_ds: dmds.DomoDataset,
    partition_key: str,
    upload_method: str,
    logger: lc.Logger,
    debug_api: bool = False,
    debug_prn: bool = False,
    debug_fn: bool = True,
    max_retry: int = 2,
    is_index: bool = False
):
    base_msg = f"{partition_key} in {consol_ds.auth.domo_instance}" if partition_key else f"in {consol_ds.auth.domo_instance}"

    if debug_fn:
        print(
            f"starting upload of {len(upload_df)} rows to {base_msg} with {max_retry} attempts")

    retry_attempt = 1
    
    res = None

    while retry_attempt <= max_retry and not res:
        try:
            if debug_fn:
                print(f"attempt {retry_attempt}/{max_retry} for {base_msg}")


            res = await consol_ds.upload_data(
                upload_df=upload_df,
                upload_method="REPLACE" if partition_key else upload_method,
                partition_key=partition_key,
                is_index=is_index,
                debug_api=debug_api,
                debug_prn=debug_prn,
            )

        except Exception as e:
            retry_attempt += 1

            message = f"⚠️ upload_data : unexpected error: {e} in {partition_key} during retry_attempt {retry_attempt}/{max_retry}"
            logger.log_warning(message)
            if debug_fn :
                print(message)
    
    if not res:
        raise Exception(f"💣 failed to upload data {len(upload_df)} rows to {base_msg} - {retry_attempt}/{max_retry} retries reached")
    
    return res

# %% ../../nbs/utils/upload_data.ipynb 4
async def upload_data(
    # instance where the data_fn function will execute against
    data_fn,  # data function to execute
    instance_auth: dmda.DomoAuth,  # instance to run the data function against
    consol_ds: dmds.DomoDataset,  # dataset where data should be accumulated
    # if partition key supplied, will replace existing partition
    partition_key: str = None,
    upload_method: str = 'REPLACE',
    is_index: bool = False,  # index dataset
    debug_prn: bool = False,
    debug_fn: bool = True,
    debug_api: bool = False,
    logger: lc.Logger = None,
    max_retry: int = 2  # number of times to attempt upload
):
    logger = logger or lc.Logger(app_name="upload_data")

    try:
        message = f"🏁 starting {instance_auth.domo_instance} - {data_fn.__name__}"
        logger.log_info(message)
        print(message)
        
        instance_session = httpx.AsyncClient()

        upload_df = await data_fn(instance_auth, instance_session, debug_api=debug_api)

        if upload_df is None or len(upload_df.index) == 0:
            message = f"no data to upload for {partition_key}: {consol_ds.id} in {consol_ds.auth.domo_instance}"
            logger.log_info(message)
            print(message)
            return None
        
        if debug_prn:
            print(upload_df[0:5])

        res = await loop_upload(
            upload_df=upload_df,
            consol_ds=consol_ds,
            partition_key=partition_key,
            upload_method=upload_method,
            debug_api=debug_api,
            debug_prn=debug_prn,
            debug_fn=debug_fn,
            max_retry=max_retry,
            logger=logger,
            is_index=False
        )

        if res.is_success:
            message = f"🚀 success upload of {partition_key} to {consol_ds.id} in {consol_ds.auth.domo_instance} in {data_fn.__name__}"
            logger.log_info(message)

        else:
            message = f"💣 upload_data successful status but failed to upload {partition_key} - {res.status} - {res.response} in {data_fn.__name__}"
            logger.log_error(message)
        
        print(message)
        
        return res

    finally:
        if is_index:

            res = await consol_ds.index_dataset(debug_api=debug_api, session=instance_session)
            if res.is_success:
                message = f"🥫 successfully indexed {consol_ds.name} in {consol_ds.auth.domo_instance}"
                logger.log_info(message)
            else:
                message = f"💀⚠️ failure to index {consol_ds.name} in {consol_ds.auth.domo_instance}"
                logger.log_error(message)
            
            print(message)

        await instance_session.aclose()

